#
# bpk_core.pl:  core bpub keywords
#
#  Include: at parse time, parses another bpub file and inserts the block 
#           where the Include keyword is. This is unlike bpub{read{"foo.bpub"}}
#           which reads and parses the file at evaluate time.
#     content: name of the bpub file
#
#  Early: at parse time, evaluates the content block and inserts the result
#         as a String where the Early keyword is.
#     content: block to be evaluated at parse time
#
#  File: save the content to an html file (returns "")
#     filename: name of the file to be saved
#     content: text of the file
#
#  Read: returns a text file as a string 
#          (looks first in the current dir, then inpath)
#     content: name of the text file
#
#  Hide: ignores all properties (useful for commenting out stuff using
#         matched brackets)
#
#  Swallow: evaluates all properties, but returns "".
#
#  Container: evaluates and returns the content property.
#
#  Time: returns the current time
#     content (optional): If "24", time is given in 24-hour time, hh:mm.
#                         If "12", time is given as hh:mm, with no am/pm.
#                         If not given, time is given as hh:mm am/pm.
#     seconds (optional): If given, returns the time for that many seconds
#                         since the epoch.  If not given, uses the current
#                         time.
#
#  Date: returns the current date
#     content (optional): Template to fill with date info.  Strings of the
#                         following characters are replaced as so:
#                         w: full name of weekday
#                         W: three letter abbreviation for weekday
#                         n: full name of month
#                         N: three letter abbreviation for month
#                         m: month number, no longer than the character string
#                         M: month number with leading zeroes, exactly as
#                            long as the character string.
#                         d/D: same thing with day number
#                         y/Y: same thing with year
#                         If not given, default is "mm/dd/yy".
#     seconds (optional): If given, returns the date for that many seconds
#                         since the epoch.  If not given, uses the current
#                         date.
#
#  Retrieve: retrieves a value from a storage file.  If the key doesn't
#            exist, "" is returned.
#     content: key to be retrieved
#     file (optional):  storage file to use.  If not given, it defaults
#                 to $Storage::defaultfile, which defaults to "_storage.bpub"
#
#  Store: stores key/value pairs into a storage file.  If the file doesn't
#         exist, it is created.
#     file (optional):  storage file to use.  If not given, it defaults
#                 to $Storage::defaultfile, which defaults to "_storage.bpub"
#     all other properties are the keys to be stored.
#
#  Unstore: deletes key/value pairs from a storage file.
#     file (optional):  storage file to use.  If not given, it defaults
#                 to $Storage::defaultfile, which defaults to "_storage.bpub"
#     all other properties are the keys to be deleted.
#
#  Makevar: creates a bpub variable local to the current block (returns "")
#     All property names are the variables to be created, and the property
#     values are the initial values of the variables
#
#  Setvar: assigns a value to a bpub variable (returns "")
#     All property names are the variables to be set, and the property
#     values are the values to set the variables to
#
#  Var: returns the value of a variable
#     content:  variable name
#
#  Macro: returns template with each "[foo]" replaced with
#          the value of the parameter "foo"
#     content (optional): the template
#     name (optional): name of variable to use as template
#       either content or name is required.
#       name="blah" is equivalent to content=var{"blah"}
#     other parameters as named in the template (in square brackets)
#
#  Perl: evaluates the content with the perl interpreter and returns the result
#          (useful for math operations and comparisons)
#     content: string to evaluate
#
#  Bpub: evaluates the content with the bpub interpreter and returns the result
#     content: string to evaluate
#
#  Repeat: evaluates the content multiple times and returns the result
#     count: number of repetitions
#     content: the stuff to be repeated
#
#  While: evaluates the content while a condition evaluates to true, and
#           returns the result
#     condition: follows perl's truth rules (zero or blank string is false)
#     content: the stuff to be repeated
#
#  If: evaluates the condition, and then evaluates and returns either 
#        the "then" or "else" property
#     condition: follows perl's truth rules (zero or blank string is false)
#     then (optional):  returned if condition is true ("" if not given)
#     else (optional):  returned if condition is false ("" if not given)
#

package Include;
@ISA = ("Keyword");


package Early;
@ISA = ("Keyword");


package String;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    return $$self{"_content"};
}


package File;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $filename = $self->getpropertytext("filename");
    my $text = $self->getpropertytext("content");
    $self->createfile($filename,$text);
    return "";
}


package Read;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    $filename = $self->getpropertytext("content");

    unless (open INFILE, $filename) {
	unless ($filename =~ /^\// or open(INFILE,$main::inpath . $filename)) {
	    $self->fatal("couldn't open file \"$filename\"");
	}
    }

    local $/ = undef;
    my $file = <INFILE>;
    close INFILE;
    return $file;
}


package Hide;
@ISA = ("Keyword");

sub gettext {
    return "";
}


package Swallow;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    foreach (keys %$self) {
	next if /^_/;
	$self->getpropertytext($_);
    }
    return "";
}


package Container;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    return $self->getpropertytext("content");
}


package Time;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($sec,$min,$hour) = localtime($self->trypropertytext("seconds")||time);
    my $format = $self->trypropertytext("content") || "";
    my $ampm = "";
    if ($format ne '24') {
	$ampm = " pm";
	if ($hour > 12) { $hour -= 12 }
	elsif ($hour < 12) { $ampm = " am";  $hour = 12 unless $hour }
    }
    if ($format eq '12') { $ampm = "" }
    $min = sprintf("%02d",$min);
    return "$hour:$min$ampm";
}


package Date;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday) =
                       localtime($self->trypropertytext("seconds") || time);
    my $weekday =
      (qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday))[$wday];
    my $month =
      (qw(January February March April May June July September),
       qw(October November December))[$mon];
    $mon++;
    $year += 1900;

    my $format = $self->trypropertytext("content") || "mm/dd/yy";
    for ($format) {
	s/(m+)/substr($mon,0-length($1))/eg;
	s/(M+)/substr(sprintf("%0".length($1)."d",$mon),0-length($1))/eg;
	s/(d+)/substr($mday,0-length($1))/eg;
	s/(D+)/substr(sprintf("%0".length($1)."d",$mday),0-length($1))/eg;
	s/(y+)/substr($year,0-length($1))/eg;
	s/(Y+)/substr(sprintf("%0".length($1)."d",$year),0-length($1))/eg;
	s/n+/$month/g;
	s/N+/substr($month,0,3)/ge;
	s/w+/$weekday/g;
	s/W+/substr($weekday,0,3)/eg;
    }
    return $format;
}


package Retrieve;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $key = $self->getpropertytext("content");
    my $file = $self->trypropertytext("file");
    return Storage::retrieve($key,$file);
}


package Store;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $file = $self->trypropertytext("file");
    foreach (keys %$self) {
	next if /^_/ or $_ eq "file";
	Storage::store($_, $self->getpropertytext($_), $file);
    }
    return "";
}


package Unstore;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $file = $self->trypropertytext("file");
    foreach (keys %$self) {
	next if /^_/ or $_ eq "file";
	Storage::unstore($_, $file);
    }
    return "";
}


package Makevar;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    foreach (keys %$self) {
	next if /^_/;
	Evaluator::makevariable($_,$self->getpropertytext($_));
    }
    return "";
}


package Setvar;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    foreach (keys %$self) {
	next if /^_/;
	unless (exists($Evaluator::vars{$_})) {
	    $self->fatal("undeclared variable \"$_\"");
	}
	$Evaluator::vars{$_} = $self->getpropertytext($_);
    }
    return "";
}


package Var;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $name = $self->getpropertytext("content");
    unless (exists($Evaluator::vars{$name})) { 
	$self->fatal("undeclared variable \"$name\"");
    }
    return $Evaluator::vars{$name};
}


package Macro;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content;
    if (exists $$self{content}) { 
	$content = $self->getpropertytext("content");
    } else {
	my $name = $self->getpropertytext("name");
	unless (exists($Evaluator::vars{$name})) {
	    $self->fatal("undeclared variable $name");
	}
	$content = $Evaluator::vars{$name};
    }
 
    $content =~ s{\[(\w+?)\]}{
	exists($$self{$1}) ? $self->getpropertytext($1) : 
	    $self->fatal("macro parameter \"$1\" not found") }eg;

    return $content;
}


package Perl;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    my $text = eval($content);
    if ($@) {
	$self->fatal("error in perl eval: $@");
    }
    return $text;
}


package Bpub;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    my $filename = $self->inheritproperty("__filename");
    my ($linenum, $charnum) = ($$self{"__linenum"}, $$self{"__charnum"});
    my $newfilename = defined $linenum ?
	"[$filename: line $linenum, char $charnum]" : "[$filename]";

    my $block = Parser::parse($content, $newfilename, $self);
    my $text = Evaluator::getblocktext($block);
    return $text;
}


package Repeat;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $count = $self->getpropertytext("count");
    my $text = "";
    while ($count--) {
	$text .= $self->getpropertytext("content");
    }
    return $text;
}


package While;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $text = "";
    while ($self->getpropertytext("condition")) {
	$text .= $self->getpropertytext("content");
    }
    return $text;
}


package If;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $text = "";
    if ($self->getpropertytext("condition")) {
	if (exists $$self{then}) { $text = $self->getpropertytext("then") }
    } else {
	if (exists $$self{else}) { $text = $self->getpropertytext("else") }
    }
    return $text;
}


1;

