#! /usr/bin/perl -w
#
# bpublish:  an automatic HTML generator (and then some)
#            by Bret Victor
#            bret@ugcs.caltech.edu
#            http://www.ugcs.caltech.edu/~bret
#

require 5.6.0;

sub help { return <<"_eot_";
bpublish v1.41 -- Bret Victor, 5/2/02                   (c) 2002 Bret Victor

usage:  bpub [options] [filename(s)]
          -i INPATH      set default directory for input files
          -o OUTPATH     set default directory for output files
          -e EVALSTRING  parse and evaluate string
          -q             quiet mode - no messages
          -m             low memory mode - no line numbers or image caching
          -v             verify mode - do not generate any output or files
          -d             debug mode - print data structure instead of output
        If filename(s) and -e are omitted, bpub reads from stdin.
        The output (if any) from the top-level block is written to stdout.

_eot_
}


# Default file paths.  Change these if you like.

$inpath = "./";
$outpath = "./";


# If FindBin module is not installed, replace the following two lines with:
#  $binpath="[PATH]"   where [PATH] is the full pathname of the directory that
#                      contains this program code.

use FindBin;
$binpath = $FindBin::Bin;
unshift @INC, $binpath;
if ($binpath !~ /\/$/) { $binpath .= "/" }


# parse command line options

%options = ();

my @evalstrings;
while (@ARGV and $ARGV[0] =~ /^-/) {
    my $opstring = shift @ARGV;
    last if $opstring eq '--';
    foreach $option (split //, substr($opstring, 1)) {
	if ($option eq '-') { last }
	elsif ($option eq 'o') {
	    $outpath = shift || '.';
	    if ($outpath !~ /\/$/) { $outpath .= "/" }
	    unless (-d $outpath) { die "outpath \"$outpath\" is an " .
				       "invalid path\n"}
	} elsif ($option eq 'i') {
	    $inpath = shift || '.';
	    if ($inpath !~ /\/$/) { $inpath .= "/" }
	    unless (-d $inpath) { die "inpath \"$inpath\" is an " .
				      "invalid path\n" }
	} elsif ($option eq 'e') { push @evalstrings, shift || "" }
	elsif ($option eq 'd') { $options{debug}  = 1 }
	elsif ($option eq 'D') { $options{debug} = $options{debugbrief} = 1 }
	elsif ($option eq 'q') { $options{quiet}  = 1 }
	elsif ($option eq 'm') { $options{lowmem} = 1;  $Image::caching = 0 }
	elsif ($option eq 'v') { $options{verify} = 1 }
	else { die help(), "unrecognized option:  -$option\n" }
    }
}


# Load in other parts of main program and keyword packages.

require "bpublish_keyword.pl";
foreach (glob "${binpath}bpk_*.pl") { require $_ }
unless (join(" ",(stat $binpath)[0,1]) eq join(" ",(stat $inpath)[0,1])) {
    foreach (glob "${inpath}bpk_*.pl") { require $_ }
}


# read some files and generate some output

unless (@ARGV or @evalstrings) { push @ARGV, "-" }
while (1) {
    my $topblock = @evalstrings && Parser::parse(shift @evalstrings, "-e") ||
                   @ARGV &&        Parser::parsefile(shift @ARGV) ||
		   last;
    unless ($options{debug}) {
	my $text = Evaluator::getblocktext($topblock);
	print $text unless $options{verify};
    } else { Evaluator::printblockdebug($topblock) }
}
Storage::closeallstorages();
exit;



# main::notify(message)
#    prints a message to stderr unless quiet mode is on.

sub notify {
    my $msg = shift;
    print STDERR "$msg\n" unless $options{quiet};
}



#--------------------------------------------------------------------------
#--- Parser                                               parses bpub code
#--------------------------------------------------------------------------
#
#  Parser::parsefile(filename, parentobject):  parse a file and return a block
#  Parser::parse(text, filename, parentobject):  parse string and return block
#  Parser::createcustomobject(parentobject, keywordname, keyvaluelist):
#                             create an object with the given property strings


package Parser;


# parsefile(filename, parentobject)
#   parses a file and returns a block (array ref) that represents it

sub parsefile {
    my ($filename, $parent, $includeobject) = @_;

    unless ($filename eq '-' or $filename =~ /^\//) {
	$filename = $main::inpath . $filename;
    }
    unless (open INFILE, $filename) {
	my $error = "couldn't open file \"$filename\"";
	if (defined $includeobject) { fatalerror($error) }
	elsif (defined $parent) { $parent->fatal($error) }
	else { fatalerror($error) }
    }
    undef $/;                                   # slurp in entire file at once
    my $file = <INFILE>;
    close INFILE;
    $/ = "\n";

    my $block = parse($file, $filename, $parent);
    main::notify("read file \"$filename\"") unless $filename eq '-';
    return $block;
}


# parse(text, filename, parentobject)
#   parses a text string and returns a block (array ref) that represents it.
#   filename string is used only for error messages; the file itself is not
#   read.

sub parse {
    local ($infile, $filename, $parent) = @_;
    local ($currentlinenum, $infilelength) = (1, length($infile));
    local @newlinepositions;

    # save newline positions so error messages can refer to proper line num
    my $newlineposition = -1;
    do { push @newlinepositions, $newlineposition }
      while (($newlineposition=index($infile,"\n",$newlineposition+1)) != -1);

    $infile =~ s{(/\*.*?\*/)}{" "x length($1)}seg;          # kill comments
    $infile .= "}";                 # pretend the file has braces around it

    my $block = accumulateblock($parent);
    foreach $member (@$block) {	$$member{"__filename"} = $filename; }
    return $block;
}


# createcustomobject (parentobject, keywordname, keyvaluelist)
#   creates and returns a keyword object from a direct list of
#   property names and string values.  This is more amenable to
#   dynamically creating objects in Perl code than the parse functions
#   are.  If a property string is undef, that property is not used.
#   If a property string begins with an underscore, the value string
#   is attached directly to the property instead of being put in a
#   keyword object in a block.  If keyvaluelist only has one entry,
#   that value is taken to be a value for the "content" property.

sub createcustomobject {
    my ($parent, $keyword, @properties) = @_;
    local $currentlinenum = -1;
    my $object = newobject($keyword,$parent);
    if (@properties == 1) { unshift @properties, "content" }
    while (@properties) {
	my ($key, $value) = splice @properties, 0, 2;
	next unless defined $value;
	$value = [ newstring($value, $object) ] unless $key =~ /^_/;
	$$object{$key} = $value;
    }
    return $object;
}


# getblock returns a block that consists of whatever single "thing" is next
#  in the file.  If the next thing is a literal string or a keyword, it will
#  be parsed and returned in a block of its own.  If the next thing is a
#  true block surrounded by braces, accummulateblock is used to slurp it up.
#  Note the mutual recursion between getblock and accumulateblock.

sub getblock {
    my ($parent) = @_;

    if (trychar("\{")) { return accumulateblock($parent) }
    if (my $block = tryquote($parent)) { return $block }

    my $keyword = getword();
    unless (defined $keyword) { fatalerror("expected quote or \{ or keyword") }
    my $object = newobject($keyword,$parent);

    if (trychar("\(")) {
	my $gotemptyproperty = 0;
	while (not trychar("\)")) {
	    my $name = getword();
	    unless (defined $name) {
		fatalerror("expected property name or \)" . 
			   ($gotemptyproperty ? " or \=" : "") );
	    }
	    if (trychar("=")) {
		$$object{$name} = getblock($object);
		$gotemptyproperty = 0;
	    } else {
		$$object{$name} = [];
		$gotemptyproperty = 1;
	    }
	}
    } elsif (trycharq("\{")) {
	$$object{"content"} = accumulateblock($object);
    } else {
	fatalerror("expected \( or \{ after keyword \"$keyword\"");
    }

    if (lc($keyword) eq 'include') {
	my $includefile = $object->getpropertytext('content');
	my $block = parsefile($includefile, $parent, $object);
	return $block;
    }
    if (lc($keyword) eq 'early') {
	my $content = $object->getpropertytext('content');
	return [ newstring($content, $parent) ];
    }

    return [ $object ];
}


# accumulateblock gets and returns a block, assuming the opening brace has
#   already been seen.  It basically uses getblock to collect "things" until
#   a closing brace is found.

sub accumulateblock {
    my ($parent) = @_;
    my @bigblock;
    while (not trychar("\}")) {
	my $subblock = getblock($parent);
	push @bigblock, @$subblock;
    }
    return \@bigblock;
}


# newstring creates a new string object

sub newstring {
    my ($string, $parent) = @_;
    my $object = newobject("String",$parent);
    $$object{"_content"} = $string;
    return $object;
}


# newobject is used by getblock to create objects, because it would be
#  awkward to use constructors.

sub newobject {
    my ($keyword, $parent) = @_;
    $keyword = "\u\L$keyword";
    unless (defined $main::{"$keyword"."::"}) { 
	fatalerror("unknown keyword $keyword", length($keyword));
    }

    my $object;
    unless ($currentlinenum == -1 or $main::options{lowmem}) {
	my ($linenum,$charnum) = getcurrentlinenumber();
	$object = { __parent => $parent,
		    __linenum => $linenum,
		    __charnum => $charnum };
    } else {
	$object = { __parent => $parent };
    }

    bless $object, $keyword;
    return $object;
}


# trychar eats whitespaces, and then returns true if the argument matches
#  the next character, false if not.  If the character matches, it is
#  eaten.

sub trychar {
    my $char = shift;
    $infile =~ s/^\s+//;
    return "" if $char ne substr($infile,0,1);
    substr($infile,0,1) = "";
    return 1;
}

# trycharq is the same as trychar, but doesn't check for whitespace.
#  It can be used if it is known that there is no whitespace, eg, immediately
#  after trychar.

sub trycharq {
    my $char = shift;
    return "" if $char ne substr($infile,0,1);
    substr($infile,0,1) = "";
    return 1;
}

# getword eats whitespace, and then returns the next word, or returns
#  undef if the next character is not a wordcharacter.  The word is eaten.
#  This is an odd implementation, but this is one of the most called
#  subroutines, and capturing parentheses in regular expressions are SLOW.
#  Using the m//g and pos trick makes it about three times as fast.

sub getword {
    $infile =~ s/^\s+//;
    if ($infile =~ m/^\w+/g) { return substr $infile, 0, pos($infile), "" }
    return undef;
}


# tryquote checks if the next character starts a quotation.  If not, it
#  returns undef.  If so, it returns a block representing the quote, with
#  ${...} constructs interpolated as appropriate.  If the quote character
#  is a double quote, all whitespace is compressed into a single space char.
#  If the quote char is a single quote, newlines are converted to spaces.
#  If the quote char is a bracket:  if the first newline is preceeded only
#  by whitespace, it and the space are removed; if the last newline is
#  followed only by whitespace, the space (but not the newline) is removed.
#  Characters escaped with a backslash are converted into whatever perl
#  thinks they should be.  The characters and the quotation marks are eaten.

sub tryquote {
    my ($parent) = @_;

    $infile =~ s/^\s+//;
    my $char = substr($infile, 0, 1);
    my $regexp = $char eq "'"  &&  qr'[\'$]'  ||
                 $char eq '"'  &&  qr'["$]'   ||
                 $char eq '['  &&  qr'[]$[]'  ||
		 return undef;
    substr($infile, 0, 1) = "";

    my $bracketdepth = 1;
    my @block;
    for ($infile) {
	while(/$regexp/g) {                 # look for a quote or dollar sign

	    # ignore if preceeded by backslash, or if dollar sign without brace
	    next if /\\.\G/ or /\$\G[^\{]/;

	    # keep track of nested brackets, and ignore closing inner brackets
	    next if $char eq '[' and ( /\[\G/ && ++$bracketdepth ||
				       /\]\G/ && --$bracketdepth );

	    # snag and kill text to this point
	    my $string = substr($_, 0, pos($_) - 1,"");

	    # take care of quote-specific text processing
	    if ($char eq '"') { $string =~ s/\s+/ /g }
	    elsif ($char eq "'") { $string =~ tr/\n/ / }
	    elsif ($char eq '[') {
		unless (@block) { $string =~ s/^[^\S\n]*\n// }
		unless (/^\$/)  { $string =~ s/\n[^\S\n]+$/\n/ }
	    }

	    # interpret backlashed characters in string
	    $string =~ s/(\\.)/"\"$1\""/eeg;

	    # create String object and push it
	    push @block, newstring($string,$parent);

	    # if we tripped on the closing quote (and not $), we're done
	    return \@block if s/^[^\$]//;

	    substr($_, 0, 2) = "";          # kill ${
	    if (s/^\s*(\w+)\s*\}//) {       # if ${ word }, kill and push Var
		push @block, createcustomobject($parent, "Var", content => $1);
	    } else {                        # otherwise, treat as block
		push @block, @{accumulateblock($parent)};
	    }
	}
    }
    $char = ']' if $char eq '[';
    fatalerror("no closing $char for quote");
}


# getcurrentlinenumber returns the line number for the current position
#   and the number of chars from the start of that line.

sub getcurrentlinenumber {
    my $charpos = $infilelength - length($infile);

    $currentlinenum++ while ($currentlinenum <= $#newlinepositions and
			     $charpos >= $newlinepositions[$currentlinenum]);
    return ($currentlinenum, $charpos - $newlinepositions[$currentlinenum-1]);
}


# fatalerror quits with the given error message, and appends a line number
# If a second argument is given, it subtracts that number (minus one) from
# the stated char position.

sub fatalerror {
    my ($msg, $charsback) = @_;
    if (defined $filename) {
	my ($linenum,$linechar) = getcurrentlinenumber();
	$linechar -= ($charsback - 1) if $charsback;
	if ($filename eq '-') { $filename = "STDIN" }
	my $location = ($linechar == 0 && $linenum > 1) ? 
	    ("end of line " . ($linenum-1)) : "line $linenum, char $linechar";
	$msg .= " at $location in file \"$filename\"";
    }
    die "$msg\n";
}



#--------------------------------------------------------------------------
#--- Evaluator                                            generates output
#--------------------------------------------------------------------------
#
#  Evaluator::getblocktext(block):  returns the output text for a block
#  Evaluator::makevariable(varname):  creates a local bpub variable
#  Evaluator::printblockdebug(block):  prints internal representation of block
#
#  %Evaluator::vars:  the bpub variables in the current block
#

package Evaluator;

%vars = ();


# getblocktext(block)
#   Goes through a block, calls gettext for each member, and returns
#   the result.

sub getblocktext {
    my ($block) = @_;
    local %localvars = ();

    my $text = "";
    foreach $member (@$block) {
	$text .= $member->gettext();
    }

    foreach $varname (keys %localvars) {
	if (defined $localvars{$varname}) { 
	    $vars{$varname} = $localvars{$varname};
	}
	else { 
	    delete $vars{$varname};
	}
    }
    return $text
}


# makevariable(varname)
# makevariable(varname,initialvalue)
#   creates a local bpub variable in the scope of the block currently
#   being traversed.

sub makevariable {
    my ($varname,$initialvalue) = @_;

    $localvars{$varname} = $vars{$varname} unless exists $localvars{$varname};
    $vars{$varname} = defined($initialvalue) ? $initialvalue : "";
}


# printblockdebug(block)
#    displays a block.

sub printblockdebug {
    my ($block,$depth) = @_;
    $depth = 0 unless defined $depth;
    unless (ref($block)) {
	unless (defined $block) { $block = "" }
	print "\"$block\"\n";
	return;
    }

    if (ref($block) !~ /^ARRAY/) {
	my ($kw) = split("=",ref($block),2);
	print "[$kw]\n";
	return;
    }
    print "\{\n";
    my $prevmember = 0;
    foreach $member (@$block) {
	if ($prevmember) {
	    print (" " x ($depth*3));
	    print "~~~\n";
	}
	unless (ref($member)) {
	    print (" " x ($depth*3));
	    print "\"$member\"\n";
	} else {
            print (" " x ($depth*3));
            print "[", ref $member, "]\n";
	    foreach $property (sort keys %$member) {
		next if $property =~ /^__/ && $main::options{debugbrief};
		print (" " x ($depth*3));
		print "$property=";
		my $value = $$member{$property};
		printblockdebug($value,$depth+1);
	    }
	}
	$prevmember = 1;
    }
    print (" " x ($depth*3));
    print "\}\n";
}


#--------------------------------------------------------------------------
#--- Storage                                      provides persistent data
#--------------------------------------------------------------------------
#
# openstorage(filename):  opens or creates a storage object
# savestorage(filename):  saves the stored data to the storage file
# closeallstorages():   saves all storage files that have been modified
# store(key,value,name): stores the given key/value pair in the storage object
# retrieve(key,name):  retrieves the value for the given key.
# unstore(key,name):  deletes the given key/value pair from the storage object


package Storage;
%storages = ();
$defaultfile = "";

# openstorage(filename)
#  if filename is not given, uses $defaultfile or "_storage.bpub"
#  opens the given storage file, or creates a new one if it doesn't exist
#  (but doesn't save it). Returns a reference to a Container keyword object
#  that holds the stored data.

sub openstorage {
    my ($name) = @_;
    $name ||= $defaultfile || "_storage.bpub";
    unless ($storages{$name}) {
	local $main::options{quiet} = 1;
	my $filename = ($name !~ /^\// and $main::inpath) . $name;
	if (-f $filename) { $storages{$name} = Parser::parsefile($name)->[0] }
	else { $storages{$name} = Parser::parse("container()")->[0] }
    }
    return $storages{$name};
}


# savestorage(filename)
#  saves the stored data to the storage file

sub savestorage {
    my ($name) = @_;
    my $storage = $storages{$name} || return;
    my $keyvaluetext = "";
    foreach (keys %$storage) {
	next if /^_/;
	my ($key, $value) = ($_, $storage->getpropertytext($_));
	$value =~ s{([\[\]])}{\\$1}g;             # escape any brackets
	$keyvaluetext .= "  $key=[$value]\n";
    }
    local $main::options{quiet} = 1;
    $storage->createfile($name, <<"_eot_", "useinpath");
/* $name:  automatically generated bpublish storage file */

container (
$keyvaluetext)
_eot_
}


# closeallstorages()
#   saves all storage files that have been modified, and frees them.

sub closeallstorages {
    foreach $name (keys %storages) {
	savestorage($name) if $storages{$name}->{_modified};
	delete $storages{$name};
    }
}


# store(key,value,name)
#   if name is not given, uses $defaultfile or "_storage.bpub"
#   stores the given key/value pair in the storage object

sub store {
    my ($key, $value, $name) = @_;
    my $storage = openstorage($name);
    $storage->addpropertytext($key, $value);
    $$storage{_modified} = 1;
}


# retrieve(key,name)
#   if name is not given, uses $defaultfile or "_storage.bpub"
#   retrieves the value for the given key.  If the key does not
#   exist, "" is returned.

sub retrieve {
    my ($key, $name) = @_;
    my $storage = openstorage($name);
    return $$storage{$key} ? $storage->getpropertytext($key) : "";
}


# unstore(key,name)
#   if name is not given, uses $defaultfile or "_storage.bpub"
#   deletes the given key/value pair from the storage object

sub unstore {
    my ($key, $name) = @_;
    my $storage = openstorage($name);
    return unless exists $$storage{$key};
    delete $$storage{$key};
    $$storage{_modified} = 1;
}



#--------------------------------------------------------------------------
#--- Image                                     determines image dimensions
#--------------------------------------------------------------------------
#
#  Image::readimageinfo(filename,imagetype):  returns width/height info
#  %Image::cache    image info cache
#  $caching         image caching enabled
#

package Image;

$caching = 1;

# readimagefile(filename, imagetype) reads the file and returns the
#  width and height of the image.  The jpg business was gleaned from
#  http://www.4guysfromrolla.com/webtech/code/imgsz.asp.html
#  so it may or may not be strictly correct.  I need to find the specs
#  one of these days...

sub readimagefile {
    my ($imagepath, $suffix) = @_;
    my ($width, $height) = (-1, -1);
    my $buf = "";
    open (IMAGE, $imagepath) || die "couldn't open file $imagepath";
    binmode IMAGE;

    if ($suffix =~ /^gif/i) {
	if ((read(IMAGE, $buf, 10) == 10) && ($buf =~ /^GIF/)) {
	    ($width, $height) = unpack("vv", substr($buf, 6));
	}
    }

    elsif ($suffix =~ /^png/i) {
	if ((read(IMAGE, $buf, 24) == 24) && ($buf =~ /^\x89PNG/)) {
	    ($width, $height) = unpack("NN", substr($buf, 16));
	}
    }

    elsif ($suffix =~ /^jpg/i) {
	local $/ = undef;
	$buf = <IMAGE>;
	my $pos = index($buf, "\xff\xd8\xff");
	if ($pos != -1) {
	    $pos += 2;
	    while (++$pos < length($buf)) {
		next if ord(substr($buf,$pos,1)) == 255;
		my ($type,$size) = unpack("Cn",substr($buf,$pos,3));
		if ($type < 192 or $type > 195)  { $pos += $size; next; }
		($height,$width) = unpack("nn",substr($buf,$pos+4,4));
		last;
	    }
	}
    }

    close IMAGE;
    return ($width, $height);
}

