#
# bpublish_keyword.pl:  methods for keyword objects
#

package Keyword;


#--------------------------------------------------------------------------
#--- Getting Property Blocks
#--------------------------------------------------------------------------
#
#  $self->getproperties($propertyname) or (@listofpropertynames)
#  $self->tryproperties($propertyname) or (@listofpropertynames)
#  $self->inheritproperties($propertyname) or (@listofpropertynames)
#  $self->tryinheritproperties($propertyname) or (@listofpropertynames)
#

# getproperties makes sure all the named properties exist, and returns
#  their values in a list.

sub getproperty { return getproperties(@_) }
sub getproperties {
    my ($self, @names) = @_;
    my @plist;
    foreach $property (@names) {
	my $block = $$self{$property};
	unless (defined $block) {
	    $self->fatal("property \"$property\" required");
	}
	push @plist, $block;
    }
    return wantarray ? @plist : $plist[0];
}


# tryproperties returns the named properties in a list, or undef for the
#  ones that don't exist.

sub tryproperty { return tryproperties(@_) }
sub tryproperties {
    my ($self, @names) = @_;
    my @plist;
    foreach $property (@names) { push @plist, $$self{$property} }
    return wantarray ? @plist : $plist[0];
}


# inheritproperties behaves like getproperties, except if it can't find
#  a property, it goes up through the __parent chain, looking for it.

sub inheritproperty { return inheritproperties(@_) }
sub inheritproperties {
    my ($self, @names) = @_;
    my @plist = $self->tryinheritproperties(@names);
    foreach $num (0..$#plist) {
	unless (defined $plist[$num]) {
	    $self->fatal("inheritable property \"$names[$num]\" required");
	}
    }
    return wantarray ? @plist : $plist[0];
}


# tryinheritproperties behaves like tryproperties, except if it can't find
#  a property, it goes up through the __parent chain, looking for it.

sub tryinheritproperty { return tryinheritproperties(@_) }
sub tryinheritproperties {
    my ($self, @names) = @_;
    my @plist;
    foreach $property (@names) {
	my $block = $$self{$property};
	my $object = $self;
	while (not defined $block) {
	    $object = $$object{"__parent"};
	    last unless ($object);
	    $block = $$object{$property};
	}
	push @plist, $block;
    }
    return wantarray ? @plist : $plist[0];
}



#--------------------------------------------------------------------------
#--- Evaluating Properties
#--------------------------------------------------------------------------
#
#  $self->getpropertytext($propertyname) or (@listofpropertynames)
#  $self->trypropertytext($propertyname) or (@listofpropertynames)
#  $self->inheritpropertytext($propertyname) or (@listofpropertynames)
#  $self->tryinheritpropertytext($propertyname) or (@listofpropertynames)
#

# getpropertytext calls the gettext method on each of the objects in the
#  property's value block, and concats and returns the result.  It also
#  works on a list of properties.

sub getpropertytext {
    my ($self, @names) = @_;
    my @values;
    foreach $property (@names) {
	my $block = $$self{$property};
	unless (defined $block) {
	    $self->fatal("property \"$property\" required");
	}
	push @values, Evaluator::getblocktext($block);
    }
    return wantarray ? @values : $values[0];
}

# trypropertytext calls the gettext method on each of the objects in the
#  property's value block, and concats and returns the result.  It also
#  works on a list of properties.  For properties that don't exist, 
#  undef is returned.

sub trypropertytext {
    my ($self, @names) = @_;
    my @values;
    foreach $property (@names) {
	my $block = $$self{$property};
	my $text = (defined $block) ? Evaluator::getblocktext($block) : undef;
	push @values, $text;
    }
    return wantarray ? @values : $values[0];
}


# inheritpropertytext behaves like getpropertytext, except if it can't find
#  a property, it goes up through the __parent chain, looking for it.

sub inheritpropertytext {
    my ($self, @names) = @_;
    my @tlist = $self->tryinheritpropertytext(@names);
    foreach $num (0..$#tlist) {
	unless (defined $tlist[$num]) {
	    $self->fatal("inheritable property \"$names[$num]\" required");
	}
    }
    return wantarray ? @tlist : $tlist[0];
}


# tryinheritpropertytext behaves like trypropertytext, except if it can't find
#  a property, it goes up through the __parent chain, looking for it.

sub tryinheritpropertytext {
    my ($self, @names) = @_;
    my @values;
    foreach $property (@names) {
	my $block = $$self{$property};
	my $object = $self;
	while (not defined $block) {
	    $object = $$object{"__parent"};
	    last unless ($object);
	    $block = $$object{$property};
	}
	my $text;
	if (defined $block) {
	    $text = "";
	    foreach $member (@$block) {
		$text .= $member->gettext();
	    }
	}
	push @values, $text;
    }
    return wantarray ? @values : $values[0];
}


#--------------------------------------------------------------------------
#--- Sharing Functionality Among Keywords
#--------------------------------------------------------------------------
#
#  $self->create($keywordname, @keyvaluelist)
#  $self->getcreatetext($keywordname, @keyvaluelist)
#  $self->morph($keywordname, $function, @args)
#  $self->getmorphtext($keyword)
#  $self->addpropertytext(@keyvaluelist)
#

# create("Keyword","propname1","propstring1","propname2","propstring2",...)
# create("Keyword","contentpropertystring")
#  creates and returns a keyword object of the given keyword with the
#  given string properties.  $self is used as the parent.  If a property
#  string is undef, that property is not used.  If a property name begins
#  with an underscore, the string is attached directly instead of being
#  put in a keyword object inside a block.

sub create {
    my ($self, $keyword, @properties) = @_;
    return Parser::createcustomobject($self, $keyword, @properties);
}


# getcreatetext( same args as create )
#   same as create(...)->gettext()

sub getcreatetext {
    my $self = shift;
    my $object = $self->create(@_);
    return $object->gettext();
}


# morph("Keyword","function",[args]) will "cast" the current object to the
#  specified keyword, call the function with the given arguments (if any),
#  cast the object back to its original keyword, and return the function's
#  result.  For example, morph("Index","gettext") will pretend that the
#  current object is an Index, and return the result of Index::gettext().
#  Useful for building one keyword's functionality on top of another, or
#  for groups of keywords that behave similarly.

sub morph {
    my ($self, $keyword, $function, @args) = @_;
    my ($result, @results); 
    my $originalname = ref $self;
    bless $self, $keyword;

    if (wantarray) { @results = $self->$function(@args) }
    else { $result = $self->$function(@args) }

    bless $self, $originalname;
    if (wantarray) { return @results }
    else { return $result }
}


# getmorphtext("Keyword") is equivalent to morph("Keyword","gettext");

sub getmorphtext {
    my ($self, $keyword) = @_;
    return $self->morph($keyword, "gettext");
}


# addpropertytext("property","value","property2","value2",etc.)
#  creates a new property that evaluates to the given string.  If
#  propertystring is omitted, the new property's block is empty.

sub addpropertytext {
    my ($self, @keyvaluelist) = @_;
    while (my ($name, $text) = splice @keyvaluelist, 0, 2) {
	if (defined $text) {
	    my $object = Parser::createcustomobject($self, "Container", $text);
	    $$self{$name} = [ $object ];
	} else {
	    $$self{$name} = [];
	}
    }
}


#--------------------------------------------------------------------------
#--- Dynamic Parsing, Block Evaluation, File I/O
#--------------------------------------------------------------------------
#
#  $self->createfile($filename, $text, $useinpath)
#  $self->parsefile($filename)
#  $self->parse($bpubcode)
#  $self->getblocktext($block)
#  $self->getparsetext($bpubcode)
#

# createfile takes a filename and some text, and creates a file containing
#  that text.  If a third parameter is passed, the inpath is used instead
#  of the outpath

sub createfile {
    return if $main::options{verify};
    my ($self, $filename, $text, $useinpath) = @_;
    my $path = $useinpath ? $main::inpath : $main::outpath;
    open (OUTFILE, ">$path$filename") or
           $self->fatal("couldn't create file \"$path$filename\"");
    print OUTFILE $text;
    close OUTFILE;
    main::notify("wrote file \"$filename\"");
}


# parsefile("filename.bpub") reads a bpub file, parses it, and returns
#   a reference to the block.  $self is used as the parent.

sub parsefile {
    my ($self, $filename) = @_;
    return Parser::parsefile($filename, $self);
}


# parse("blocktext") parses the text as a bpub block and returns
#   a reference to the block.  $self is used as the parent.
#   If a second argument is given, it is used as the filename for
#   error purposes.

sub parse {
    my ($self, $blocktext, $filename) = @_;
    unless (defined $filename) {
	my ($fname, $line) = (caller())[1..2];
	$filename = "[$fname: line $line]";
    }
    return Parser::parse($blocktext, $filename, $self);
}


# getblocktext($block) evaluates a block;

sub getblocktext {
    my ($self, $block) = @_;
    return Evaluator::getblocktext($block);
}


# getparsetext("blocktext") parses the text and evaluates the block.

sub getparsetext {
    my $self = shift;
    my $block = $self->parse(@_);
    return Evaluator::getblocktext($block);
}


#--------------------------------------------------------------------------
#--- Dealing with Images
#--------------------------------------------------------------------------
#
#  $self->tryimageinfo($imagename)
#  $self->getimageinfo($imagename)
#  $self->tryimages(@listofimagenames)
#  $self->getimages(@listofimagenames)
#

# tryimageinfo(imagename)
#   returns the suffix, width and height of a jpg, gif or png file, or
#   false if the image doesn't exist.  The name should be given without
#   a suffix.

sub tryimageinfo {
    my ($self,$imagename) = @_;

    my $imagepath = (($imagename !~ /^\//) ? $main::outpath : "") . $imagename;
    if ($Image::caching and $Image::cache{$imagepath}) {
	return @{$Image::cache{$imagepath}}
    }

    my ($suffix, $width, $height) = ("", 0, 0);
    foreach $suf (qw(jpg gif png)) {
	if (-f ("$imagepath.$suf")) {
	    ($width, $height) = Image::readimagefile("$imagepath.$suf", $suf);
	    if ($width == -1) {
		$self->fatal("could not determine dimensions of " .
			     "image \"$imagename.$suf\"");
	    }
	    $suffix = $suf;
	    last;
	}
    }
    $Image::cache{$imagepath} = [ $suffix,$width,$height ] if $Image::caching;
    return ($suffix, $width, $height);
}


# getimageinfo(imagename)
#   does the same as tryimageinfo, but dies if the image doesn't exist

sub getimageinfo {
    my $self = shift;
    my ($suffix,$width,$height) = $self->tryimageinfo(@_);
    return ($suffix,$width,$height) if $suffix;
    $self->fatal("image \"$_[0].xxx\" can't be found");
}


# tryimages(imagename1, ...)
#   returns a hashref for each name passed.  The hash contains the
#   the keys "name", "suffix", "width", and "height".  Unavailable images
#   are returned as undef.

sub tryimages {
    my $self = shift;
    my @arrayofhashes;
    foreach (@_) {
	my $imagename = $_;
	1 while $imagename =~ s{[^/]+/\.\./}{};     # collapse ../ directories
	my ($suffix,$width,$height) = $self->tryimageinfo($imagename);
	push @arrayofhashes, $suffix ? { 'name' => "$imagename.$suffix",
					 'suffix' => $suffix,
					 'width' => $width,
					 'height' => $height } : undef;
    }
    return wantarray ? @arrayofhashes : $arrayofhashes[0];
}


# getimages(iamgename1, ...)
#   is the same as tryimages, but it dies if an image is not available.

sub getimages {
    my $self = shift;
    my @arrayofhashes = $self->tryimages(@_);
    foreach (@arrayofhashes) {
	$self->fatal("image $_[0].xxx can't be found") unless $_;
	shift;
    }
    return wantarray ? @arrayofhashes : $arrayofhashes[0];
}


#--------------------------------------------------------------------------
#--- Dealing with Prefixes
#--------------------------------------------------------------------------
#
#  $self->getprefix()
#  $self->getuprefix()
#  $self->getsprefix()
#

# getprefix makes a prefix string by traversing the heirarchy, starting
#  from the parent of the current object.  If an object is found with
#  the package variable $prefixproperty defined, then the value of the
#  property by that name is added to the prefix.  If an object has
#  has a "prefix" property defined, that value is added to the prefix
#  and the process stops.
#  Two strings are returned, one using underscores as separators
#  and one using slashes, both with trailing separators.

sub getprefix {
    my ($self, $prefixtype) = @_;
    my $pfx = "";
    my $object = $self;
    while (defined $object) {
	if ($object != $self) {
	    my $pfxprop = $ { ref($object) . "::prefixproperty" };
	    if (defined $pfxprop) {
		my $name = $object->inheritpropertytext($pfxprop);
		$pfx = "${name}_$pfx" unless $name eq "";
	    }
	}
	if (exists $$object{prefix}) {
	    my $pre = $object->getpropertytext("prefix");
	    $pre =~ s[/][_]g;
	    unless ($pre =~ /_$/) { $pre .= "_" }
	    $pfx = $pre . $pfx;
	    last;
	}
	$object = $$object{"__parent"};
    }
    my $spfx = $pfx;
    $spfx =~ s[_][/]g;

    $prefixtype ||= "";
    if ($prefixtype eq 'u') { return $pfx }
    elsif ($prefixtype eq 's') { return $spfx }
    return ($pfx,$spfx);
}


# getuprefix() is the same as getprefix('u')

sub getuprefix {
    my ($self) = @_;
    return $self->getprefix('u');
}


# getsprefix() is the same as getprefix('s')

sub getsprefix {
    my ($self) = @_;
    return $self->getprefix('s');
}


#--------------------------------------------------------------------------
#--- Dealing with Subproperties
#--------------------------------------------------------------------------
#
#  $self->attachsubpropertytext($contentpropertyname,@listofsubpropertynames)
#  $self->getsubpropertytextlist($contentpropertyname,@listofsubpropertynames)
#  $self->getsubpropertytexthash($contentpropertyname,@listofsubpropertynames)
#

# attachsubpropertytext calls getpropertytext on each given subproperty
#  and attaches each object's text to the object under the name of the
#  subproperty with an uderscore in front.  That didn't make any
#  sense, did it?
#  If the underscored property already exists, it is left alone.

sub attachsubpropertytext {
    my ($self, $contentproperty, @names) = @_;

    my $contentblock = $self->getproperties($contentproperty); 
    foreach $member (@$contentblock) {
	foreach $property (@names) {
	    unless (exists $$member{"_$property"}) {
		$$member{"_$property"} = $member->getpropertytext($property);
	    }
	}
    }
}


# getsubpropertytextlist returns a list of list references, one for each
#  subproperty given.  Each reference points to a list of strings, one
#  for each member of the content block.  Note that this ordering is
#  opposite of getpropertytexthash, below.  The arguments are the name
#  of the content property, and then the names of the subproperties.

sub getsubpropertytextlist {
    my ($self, $contentproperty, @names) = @_;

    my $contentblock = $self->getproperties($contentproperty);
    my @listoflists;
    foreach $property (@names) {
	my $propertylist = [];
	foreach $member (@$contentblock) {
	    my $mp = $member->getpropertytext($property);
	    push @$propertylist, $mp;
	}
	push @listoflists, $propertylist;
    }
    return wantarray ? @listoflists : $listoflists[0];
}


# getsubpropertytexthash returns a list of hash references, one for each
#  member of the content block.  Each reference points to a hash with
#  keys corresponding to the requested subproperties and values of
#  text strings.

sub getsubpropertytexthash {
    my ($self, $contentproperty, @names) = @_;

    my $contentblock = $self->getproperties($contentproperty); 
    my @listofhashes;
    foreach $member (@$contentblock) {
	my $memberhash = {};
	foreach $property (@names) {
	    my $mp = $member->getpropertytext($property);
	    $$memberhash{$property} = $mp;
	}
	push @listofhashes, $memberhash;
    }
    return wantarray ? @listofhashes : $listofhashes[0];
}


#--------------------------------------------------------------------------
#--- Error Handling
#--------------------------------------------------------------------------
#
#  $self->notify($message)
#  $self->fatal($message)
#

# notify("message") prints the message to stderr unless quiet mode is on.
#  If the message does not end with a newline, keyword, filename, and
#  line/char info are appended.

sub notify {
    my ($self, $msg) = @_;
    return if $main::options{quiet};
    unless ($msg =~ /\n$/) {
	my ($keyword) = split("=",ref($self),2);
	my $linenum = $$self{"__linenum"};
	my $charnum = $$self{"__charnum"};
	$charnum -= (length($keyword) - 1) if $keyword;
	my $filename = $self->tryinheritproperty("__filename") || "";
	if ($filename eq '-') { $filename = "STDIN" }
	$msg .= " at keyword \"$keyword\"" if defined $keyword;
	if (defined $linenum) {
	    $msg .= ($charnum == 0) ? (" at end of line " . ($linenum - 1))
	      : " at line $linenum, char $charnum";
	}
	$msg .= " in file \"$filename\"" if $filename;
	$msg .= "\n";
    }
    print STDERR $msg;
}


# fatal("message") prints the message to stderr and dies.
#  If the message does not end with a newline, keyword, filename, and
#  line/char info are appended.

sub fatal {
    my ($self, $msg) = @_;
    $main::options{quiet} = 0;
    $self->notify($msg);
    exit 1;
}


# AUTOLOAD is called when an undefined method is called.  It prints an
#  error message.

sub AUTOLOAD {
    my $self = shift;
    my $method = $AUTOLOAD;
    if ($method eq lc($method)) {
	$self->fatal("method \"$method\" undefined");
    } elsif ($method !~ /::[A-Z]+$/) {
	die "undefined method $method\n";
    }
}


1;

