#
# bpk_blawg.pl:  bpub keywords for blawg and update log
#
#  Blawg:  generates the blawg entries and index, and returns nothing.
#          It works by morphing into an Index, so tricks that work with
#          Index items may work here as well.
#     content: a block of Entries
#     name: name of the blawg
#     title (inheritable): title for the blawg
#     intro (inheritable): text to be placed at the top of the index
#
#  Entry:  container for a blawg entry
#     date: entry date, in mm/dd/yy format
#     content (optional): text for blawg entry
#     music (optional): text for mlawg entry
#
#  Updatelog:  generates the updatelog, and returns nothing.
#     content: a block of Updates
#     name: name of the updatelog
#     title: title for the updatelog
#     blawg (optional): a Blawg to merge with the updates.
#     daysback (optional): number of days before today to start the log.
#                          If not given, defaults to 60.
#
#  Update:  container for updates in Updatelog
#     date: date of update in mm/dd/yy format
#     content: text of the update
#

package Blawg;
@ISA = ("Keyword");
$prefixproperty = "name";
use Time::Local;

sub gettext {
    my ($self) = @_;
    my $name = $self->getpropertytext("name");
    my $contentblock = $self->getproperty("content");
    my ($firstdate,$firstname);

    foreach $entry (@$contentblock) {
	my $date = $entry->getpropertytext("date");
	$firstdate ||= $date;
	my ($mon,$day,$year) = split("/",$date);
	my $fullmonth = (qw/January February March April May June July August/,
			 qw/September October November December/)[$mon-1];
	my $entryname = "${mon}_${day}_${year}";
	$firstname ||= $entryname;
	$entry->addpropertytext("name", $entryname);
	$entry->addpropertytext("title", "$fullmonth $day, 20$year");

	my $daynum=(localtime(timelocal(0,0,12,$day,$mon-1,$year+100)))[6];
	$$entry{_dayname} = (qw/Sunday Monday Tuesday Wednesday Thursday/,
			    qw/Friday Saturday Sunday/)[$daynum];

	my $thumbnail = $$entry{music} && $$entry{content} && "both" ||
                       	$$entry{music} && "music" || "text";
	$entry->addpropertytext("thumbnail", $thumbnail);

	$entry->addpropertytext("blurb", $entry->getblurb());
    }

    $self->addpropertytext("nochooserballs");
    $self->addpropertytext("swaparrowlabels");
    my $index = $self->getmorphtext("Index");

    my $uprefix = $self->getuprefix();
    my $filename = $uprefix . $name;
    my $file = $self->getcreatetext("Htmlfile",
				     content => $index,
				     redirect => $filename);
    $self->createfile("$filename.html",$file);

    $self->getcreatetext("Store",
			 "latestblawg" => $firstdate,
			 "latestblawglink" => "$uprefix${name}_$firstname" );
    return "";
}


package Entry;
@ISA = ("Keyword");

sub getblurb {
    my ($self) = @_;
    my $blurb = "...";
    if ($$self{content}) {
	$blurb = $self->getpropertytext("content");
	$blurb =~ s/<.*?>//sg;
	if ($blurb =~ /^\s*(.+?[.?!]+)(.+?[.?!])/s) {
	    $blurb = $1 . $2;
	    if (length($blurb) > 160) { $blurb = $1 }
	    $blurb .= "...";
	} else { $blurb = "..."}
    } elsif ($$self{music}) {
	$blurb = $self->getpropertytext("music");
	$blurb =~ s/<.*?>//sg;
	if ($blurb =~ /^\s*(.+?[.?!])/s) { $blurb = $1 }
	else { $blurb = "..." }
    }
    return $blurb;
}

sub gettext {
    my ($self) = @_;
    my ($uprefix, $sprefix) = $self->getprefix();
    my $font = $self->getcreatetext("Font");

    my ($date) = $self->getpropertytext("date");
    my $dayname = $self->getproperty("_dayname");

    my ($text,$music,$spacer) = ("","","");
    if ($$self{content}) {
	$text = $self->getcreatetext("Makepars",
				     $self->getpropertytext("content"));
	my $textimg = $self->getimages("images/${sprefix}text");
	$text = <<"_eot_";
<tr>
<td width="$$textimg{width}" align="left" valign="top"><img
  src="$$textimg{name}" width="$$textimg{width}"
  height="$$textimg{height}" border="0"></td>
<td width="10"><img src="images/etc/spacer.gif" width="10" height="2"></td>
<td valign="top">
$font
$text
</font></td></tr>
_eot_
    }
    if ($$self{music}) {
	$music = $self->getcreatetext("Makepars",
				      $self->getpropertytext("music"));
	my $musicimg = $self->getimages("images/${sprefix}music");
	$music = <<"_eot_";
<tr>
<td width="$$musicimg{width}" align="left" valign="top"><img
  src="$$musicimg{name}" width="$$musicimg{width}"
  height="$$musicimg{height}" border="0"></td>
<td width="10"><img src="images/etc/spacer.gif" width="10" height="2"></td>
<td valign="top">
$font
$music
</font></td></tr>
_eot_
    }

    if ($music and $text) {
	$spacer = "<tr><td colspan=\"3\"><br></td></tr>\n";
    }

    return <<"_eot_";
<p>$dayname, $date</p>
<p><table cellspacing="0" cellpadding="0" border="0" width="100%">
$text
$spacer
$music
</table></p>
_eot_
}


package Updatelog;
@ISA = ("Keyword");
$prefixproperty = "name";
use Time::Local;

sub gettext {
    my ($self) = @_;
    my ($name,$title) = $self->getpropertytext(qw(name title));
    my $contentblock = $self->getproperty("content");
    my $daysback = $self->trypropertytext("daysback") || 60;
    my $blawgblock = $$self{blawg} ?
      $self->getproperty("blawg")->[0]->getproperty("content") : [];
    my $font = $self->getcreatetext("Font");

    my $count = 0;
    my %entries;

    my $stopseconds = time - $daysback * 24 * 60* 60;
    foreach $entry (@$contentblock, @$blawgblock) {
	my ($mon,$day,$year) = split '/', $entry->getpropertytext("date");
	my $seconds = timelocal(0,0,12,$day,$mon-1,$year+100) - $count;
	next if $seconds < $stopseconds;
	$entries{sprintf("%010d",$seconds)} = $entry;
    } continue { $count++ }

    my $text = $self->getcreatetext("Heading", title => $title);
    $text .= '<p><table cellspacing="0" cellpadding="0" border="0">';

    my $firstdate;
    foreach (reverse sort keys %entries) {
	my $entry = $entries{$_};
	my $date = $entry->getpropertytext("date");
	$firstdate ||= $date;
	$text .= <<"_eot_";
<tr><td nowrap align="right" valign="top">$font<b>$date:</b>&nbsp; &nbsp;
</font></td><td valign="top">$font
_eot_
	if (ref $entry eq 'Update') {
	    $text .= $entry->getpropertytext("content");
	} elsif (ref $entry eq 'Entry') {
	    my $blurb = $entry->trypropertytext("blurb") || $entry->getblurb();
	    my ($mon,$day,$year) = split("/",$date);
	    my @parts;
	    push @parts, "blawg" if $$entry{content};
	    push @parts, "mlawg" if $$entry{music};
	    $text .= $entry->getcreatetext("Locallink",
					   link => "${mon}_${day}_${year}",
					   content => join(" and ", @parts),
					   usepfx => 1 );
	    $text .= " entr". (@parts > 1 ? "ies" : "y") . ". <i>($blurb)</i>";
	}
	$text .= <<"_eot_";
</font></td></tr>
<tr><td colspan="2" height="16"><img src="images/etc/spacer.gif"
                                    width="2" height="16"></td></tr>
_eot_
    }
    $text .= "</table></p>\n";

    my $uprefix = $self->getuprefix();
    my $filename = $uprefix. $name;
    my $file = $self->getcreatetext("Htmlfile",
				    content => $text,
				    redirect => $filename);
    $self->createfile("$filename.html",$file);
    $self->getcreatetext("Store", "latestupdate" => $firstdate);


    ###  create splash_updates.html

    my $splashtext = <<"_eot_";
<table width="100%" height="100%" border="0" cellspacing="0" cellpadding="0">
<tr><td align="right" valign="bottom">
_eot_
    my $splashmsg = $self->getcreatetext("Font",
					 size => "-2",
					 color => "#c0c0c0",
					 content => "Last update: $firstdate");
    $splashtext .= $self->getcreatetext("Switchlink",
					nounderline => 1,
					link => $filename,
					subtext => "Recent updates",
					content => $splashmsg );
    $splashtext .= "</td></tr></table>\n";

    my $splashfile = $self->getcreatetext("Htmlfile", body => $splashtext);
    $self->createfile("splash_updates.html", $splashfile);

    return "";
}

package Update;
@ISA = ("Container");


1;
