#
# bpk_link.pl:  bpub keywords related to linking.  Unless stated otherwise,
#               all keywords here support the subtext, target, and newwindow
#               properties, since they all use Link to do the dirty work.
#
#  Link: granddaddy link keyword.  Returns an <A> tag, with a closing
#        tag if content is given.
#     href:  href for the link
#     subtext (optional):  mouseover status message
#     mouseover (optional):  javascript for mouseover event
#     mouseout (optional):  javascript for mouseout event
#     other (optional):  any other tags or script to stick into the link
#     target (optional):  frame/window target for the link
#     newwindow (boolean):  same as target="_blank"
#     nounderline (boolean):  means that link content won't be underlined
#     content (optional):  text of the link.  If not given, just the <A> tag
#                          is returned, with no closing tag.
#
#  Extlink:  returns a link to a url that will open up in a new window
#     href:  link url
#     content:  text of the link
#
#  Extfile:  returns a link to a url that is presumably a downloadable file
#     href:  link url
#     content:  text of the link
#
#  Toplink:  returns a link to a url that will open in the top frame
#     href:  link url
#     content:  text of the link
#
#  Switchlink:  returns a link to switch to another top-level index and
#               open up the given file inside the frame.  The prefix of
#               the link property determines which index to switch to.
#     content: the text of the link
#     link: page name
#
#  Locallink: returns a link to an html file on this site
#     content: the text of the link
#     link: page name.  Unless it ends with "/" or a suffix, an ".html"
#           suffix will be added.
#     usepfx (boolean): if not given, link is a full page name with
#                       underscores.  If given, link is taken relative to
#                       the current prefix
#
#  Localfile: returns a link to a file in the images directory
#     content: the text of the link
#     file: name of the file
#     usepfx (boolean): if not given, file is a full pathname relative to
#                       the images directory.  If given, file is relative
#                       to the current prefix
#
#  Filewindow: same as localfile(usepfx newwindow)
#     content: the text of the link
#     file: name of the file, relative to the current prefix
#
#  Picwindow: same as filewindow, but for images
#     content: the text of the link
#     img: name of the img with no suffix, relative to the current prefix
#
#  Iigsdownload: returns a link to download a IIgs file (no Link properties)
#     content: local name of the file, or "" to link no file
#     size (optional): size of the file in Kb.  If not given, the filesize
#                      in determined by examining the file.
#     docname (optional) : name of the documentation file, minus the suffix
#                          if not given, the name property is used
#     nodocs (optional) : if specified, no documentation is linked
#     server (optional) : keyname for an alternate server.  If not given,
#                         the file is assumed to be local
#
#  Musiclink: returns a link to download a mlawg music file (no Link props)
#     content: text of the link
#     lofi (optional): filesize in Kb of low quality version
#     lofirate (optional): bitrate of low quality version in Kbps
#     hifi (optional): filesize in Kb of high quality version
#     date (inheritable): used to form the filename to download
#     file (optional): filename of music file, minus ".mp3" suffix
#

package Link;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $href = $self->getpropertytext("href");
    my ($subtext, $mouseover, $mouseout, $other, $target, $content) =
      $self->trypropertytext(qw(subtext mouseover mouseout other),
			     qw(target content));

    my $style = $$self{nounderline} ? ' style="text-decoration:none"' : "";
    if ($$self{newwindow}) { $target = "_blank" }
    if ($subtext) {
	$subtext =~ s{'}{\\'}g;
	$mouseout .= ";window.status=' '";
	$mouseover .= ";window.status='$subtext'";
    }
    $target = qq( target="$target") if $target;
    $other = " $other" if $other;
    $mouseout = qq( onMouseOut="$mouseout;return true;") if $mouseout;
    $mouseover = qq( onMouseOver="$mouseover;return true;") if $mouseover;
    $href = qq( href="$href");

    my $link = "<a$style" . ($other||"") . ($mouseout||"") .
                            ($mouseover||"") . ($target||"") . "$href>";
    $link .= "$content</a>" if defined $content;
    return $link;
}


package Extlink;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($content) = $self->getpropertytext("content");
    $self->addpropertytext("newwindow");
    return $self->getmorphtext("Link");
}


package Extfile;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($content) = $self->getpropertytext("content");
    return $self->getmorphtext("Link");
}


package Toplink;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($content) = $self->getpropertytext("content");
    $self->addpropertytext("target", "_top");
    return $self->getmorphtext("Link");
}


package Switchlink;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $link = $self->getpropertytext("link");
    my ($index) = ($link =~ /^(\w+?)_/);
    $self->addpropertytext("href", "${index}_index.html?$link");
    return $self->getmorphtext("Toplink");
}


package Locallink;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($link,$content) = $self->getpropertytext(qw(link content));
    $link = $self->getuprefix() . $link if $$self{usepfx};
    $link .= ".html" unless $link =~ /(\/|\..{2,4})$/;
    $self->addpropertytext("href", $link);
    return $self->getmorphtext("Link");
}


package Localfile;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($file,$content) = $self->getpropertytext(qw(file content));
    $file = $self->getsprefix() . $file if $$self{usepfx};
    $self->addpropertytext("href", "images/$file");
    return $self->getmorphtext("Link");
}


package Filewindow;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    $self->addpropertytext("newwindow");
    $self->addpropertytext("usepfx");
    return $self->getmorphtext("Localfile");
}


package Picwindow;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $img = $self->getimages("images/" . $self->getsprefix() .
			       $self->getpropertytext("img"));

    $self->addpropertytext("href", $$img{name});
    $self->addpropertytext("newwindow");
    return $self->getmorphtext("Link");
}




package Iigsdownload;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($uprefix,$sprefix) = $self->getprefix();
    my $name = $self->inheritpropertytext("name");
    my ($file) = $self->getpropertytext("content");
    my $filename = $file;

    my $longname;
    my $server = $self->trypropertytext("server");
    unless ($server) {
	$longname = "images/$sprefix$filename";
    } elsif ($server eq 'cs') {
	$longname="http://www.cs.caltech.edu/~bret/iigs/$filename";
    } else { $self->fatal("invalid server keyword $server") }

    my $text = "";

    my $docname = "images/$sprefix" . ($self->trypropertytext("docname") ||
				       $name) . ".html";
    unless ($$self{nodocs} or (not -f "$main::outpath$docname")) {
	my $doclink = $self->trypropertytext("doclink") ||
	              "Read the documentation";
	$text .= qq(<a href="$docname">$doclink</a><br>);
    }

    unless ($file) { return $text }

    my $size;
    if ($$self{server} or $$self{size}) {
	$size = $self->getpropertytext("size");
    } else {
	my @filestat = stat("$main::outpath$longname") or
	    $self->fatal("file $longname not found");
	$size = int($filestat[7] / 1024) + 1;
    }

    $text .= qq(<a href="$longname">Download $file ($size Kb)</a>);
    return $text;
}


package Musiclink;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    my ($lofi, $hifi) = $self->trypropertytext(qw(lofi hifi));
    my $lofirate = $self->trypropertytext("lofirate") || "32";

    my $file = $self->trypropertytext("file");
    unless ($file) {
	my $date = $self->inheritpropertytext("date");
	my ($mon,$day,$year) = split("/",$date);
	if ($mon < 10) { $mon = "0$mon" }
	if ($day < 10) { $day = "0$day" }
	$file = "bret_${mon}_${day}_$year";
    }

    my $hrefpfx = "http://www.cs.caltech.edu/~bret/mlawg/$file";
    my $hreflofi = $hrefpfx . ".mp3";
    my $hrefhifi = $hrefpfx . "_hifi.mp3";

    if ($lofi) {
	my $seconds = int(($lofi * 8 * 1024) / ($lofirate * 1000));
	my $time = ($seconds < 60) ? "$seconds seconds" : 
	    int($seconds/60).":".(($seconds%60 < 10) ? "0" : "") . $seconds%60;
	my $link = qq(<a href="$hreflofi">$content ($time, ${lofi}Kb)</a>);
	$link .= qq( or <a href="$hrefhifi">(${hifi}Kb)</a>) if $hifi;
	return $link;
    } else {
	return qq(<a href="$hreflofi">$content</a>);
    }
}



1;



