#
# bpk_text.pl:  bpub keywords for text formatting and basic HTML stuff
#
#  Font:  returns a <FONT> tag, with a closing tag if content is given.
#         All properties are optional.  If an attribute property is not
#         given, it defaults to the standard.  If an attribute property
#         is given as "", it is not included in the tag at all.
#    name: predefined font name.  The attributes of the predefined font
#          can be overriden using the other properties
#    face: font face
#    size: font size
#    color: font color
#    content: text to be fontified.  If not given, just the FONT tag is
#             returned, with no closing tag.
#
#  Makepars:  places html <p></p> tags around paragraphs.
#    content: text to be paragraphed
#    align (optional):  ALIGN attribute for <P> tags
#    separator (optional):  paragraph separator.  If not given, splits
#                           on one or more blank lines.
#
#  Text:  makes the first letter of each paragraph big.  Runs the text
#         through Makepars, so use blank lines (or par()) as your
#         paragraph separator.
#    content: the text to be formatted
#
#  Heading: places a heading picture, or a text heading if there is no picture
#     name (inheritable): name of the section
#     title (inheritable): text heading, if there is no picture
#
#  Par: inserts a paragraph break (double newline)
#
#  Br: inserts a line break
#
#  Applet: places a java applet
#     code: name of the java class
#     width: width of the applet
#     height: height of the applet
#
#  Center: centers the text
#     content: the text to be centered
#
#  Bold: makes the text bold
#     content: the text to be enboldened
#
#  Ital: makes the text italicized
#     content: the text to be italicized
#
#  Hilite: makes the text a different color
#     content: the text to be hilited
#
#  Smallfont: makes the text use a tiny font size
#     content: the text to be shrunk
#
#  Numberedlist: places the items in an ordered list
#     content: a block of Listitems
#
#  Listitem: contains an entry in a Numberedlist
#     content: the text of the item
#
#  Simpletable: places an html table with border=0
#     content: a flat block of Items (or whatever) containing the table entries
#     rows: width of the table
#
#  Box: draws a box around the content
#     content: stuff to be boxed
#

package Font;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($face,$size,$color,$name,$content) =
      $self->trypropertytext(qw(face size color name content));

    $name ||= "";
    if ($name eq 'heading') { $size = "+1";  $color = "#808080" }
    elsif ($name eq 'smallheading') { $color = "#808080" }

    $face =  "arial,verdana,sans-serif" unless defined $face;
    $size =  "-1" unless defined $size;
    $color = "#000000" unless defined $color;

    $face =  qq( face="$face") if $face;
    $size =  qq( size="$size") if $size;
    $color = qq( color="$color") if $color;

    my $font = "<font$face$size$color>";
    $font .= "$content</font>" if defined $content;
    return $font;
}


package Makepars;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    my $align = $self->trypropertytext("align") || "";
    $align = qq( align="$align") if $align;           # best statement ever!

    my @pars;
    unless ($$self{separator}) {
	$content =~ s/^\n+//;
	$content =~ s/\n+$//;
	@pars = split /( *\n){2,} */, $content;
    } else { @pars = split $self->getpropertytext("separator"), $content }

    my $text = "";
    foreach (@pars) {
	if (/^\s*<[pP][\s>]/) { $text .= "$_\n" }
	elsif (/\S/) { $text .= "<p$align>$_</p>\n" };
    }
    return $text;
}



package Text;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getmorphtext("Makepars");
    my $parcount = 0;

    $content =~ s{ <[pP]> \s* ((<\w>)*) \s* (\w) (\S*) }{
	my $img = $self->getimages("images/alphabet/bigletter_\l$3");
	qq(<p>$1<img src="$$img{name}" width="$$img{width}") .
	qq( height="$$img{height}" valign="baseline">) .
	  ($parcount++ ? $4 : "\U$4") . "\n";
     }egx;

    return $content;
}


package Heading;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $uprefix = $self->getuprefix();
    $uprefix =~ s/_$//;
    my $headingimg = $self->tryimages("images/headings/heading_$uprefix");

    if ($headingimg) { return <<"_eot_" }
<p><img src="$$headingimg{name}" width="$$headingimg{width}"
        height="$$headingimg{height}" border="0"></p>
_eot_

    my $headingfont = $self->getcreatetext("Font", name => "heading");
    my $title = $self->inheritpropertytext("title");
    return "<p>$headingfont<i>$title</i></font></p>\n";
}


package Applet;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($code,$width,$height) = $self->getpropertytext(qw(code width height));

    return <<"_eot_";
<center><applet code=\"$code\" width=$width height=$height></applet></center>
_eot_
}

package Par;
@ISA = ("Keyword");
sub gettext { return "\n\n" }

package Br;
@ISA = ("Keyword");
sub gettext { return "<br>\n" }

package Center;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    return qq(\n\n<p align="center">$content</p>\n);
}

package Bold;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    return "<b>$content</b>";
}

package Ital;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    return "<i>$content</i>";
}

package Hilite;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    return "<font color=\"#a00000\">$content</font>";
}

package Smallfont;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my $content = $self->getpropertytext("content");
    return "<font size=\"-3\">$content</font>";
}

package Numberedlist;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($contentblock) = $self->getproperties("content");
    my $text="<ol>\n";
    foreach $item (@$contentblock) {
	my $itemtext = $item->getpropertytext("content");
	$text .= "<li>$itemtext</li>\n";
    }
    $text .= "</ol>\n";

    return $text;
}

package Listitem;
@ISA = ("Container");


package Simpletable;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($contentblock) = $self->getproperties("content");
    my ($columns) = $self->getpropertytext("columns");
    my $font = $self->getcreatetext("Font");

    my $text="\n\n<p><table border=0>";
    my $count = 0;
    foreach $item (@$contentblock) {
	my $itemtext = $item->getpropertytext("content");
	$text .= "<tr>" if $count == 0;
	$text .= "<td valign=\"baseline\">$font$itemtext</font></td>\n";
	$count++;
	if ($count >= $columns) {
	    $text .="</tr>";
	    $count = 0;
	}
    }
    unless ($count == 0) { $text .= "</tr>" }
    $text .= "</table></p>\n\n";

    return $text;
}

package Box;
@ISA = ("Keyword");

sub gettext {
    my ($self) = @_;
    my ($content) = $self->getpropertytext("content");
    my $font = $self->getcreatetext("Font");

    return <<"_eot_";
\n\n<p><table border="1" width="100%" cellspacing="0"><tr><td width="100%">
$font$content
</font></td></tr></table></p>\n
_eot_
}


1;
